
 
//#define F_CPU 8000000UL              
#include <avr/io.h>                      
#include <util/delay.h>
#include <stdlib.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
 
#define F_OSC 18432000                
#define BAUD 115200 //38400                
//#define MYUBRR 9 //#define MYUBRR FOSC/16/BAUD-1
#define MYUBRR (F_OSC/16/BAUD - 1) 
 

#define ADC_maxchannel 6
volatile unsigned ADC_recent_value[ADC_maxchannel]={0}; 
volatile unsigned char ADC_channel=0; 

#define ADC_avg_max 10
volatile unsigned ADC_avg_value[ADC_maxchannel][ADC_avg_max];
volatile unsigned char ADC_avg_pos=0;

#define GLOVE_STATE_IDLE  0
#define GLOVE_STATE_QUERY 1
#define GLOVE_STATE_QUERY_N 2
#define GLOVE_STATE_STREAM  3
unsigned char glove_state = GLOVE_STATE_IDLE;

void USART_Init()
{
 
UBRR0H = (unsigned char)(MYUBRR>>8);     
UBRR0L = (unsigned char)MYUBRR;            
 
UCSR0B = (1<<RXEN0)|(1<<TXEN0);           // Rx Tx enable
 
UCSR0C = (3<<UCSZ00);                     // Set frame format: 8data, 1 stop bit
 
}



void USART_Transmit( unsigned char data )
{
 
while ( !( UCSR0A & (1<<UDRE0)) );        
 
UDR0 = data;
                          
 
}
 

 
unsigned char USART_Receive( void )
{
 
while ( !(UCSR0A & (1<<RXC0)) );         
 
 
return UDR0;                             
}


void USART_Transmitbuf(char * data, int n) {
	for(int i = 0; i < n; i++) {
		USART_Transmit(data[i]);
	}
}

void ADC_Init() {


     //ADMUX |= (1 << REFS0) | (1 << REFS1); // 11 = 1,1V 
   //ADMUX |= (1 << ADLAR); // left adjustment 
   //ADCSRA |= (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); // prescaler = 128 -> 125kHz 
   //ADCSRA |= (1 << ADATE); // adc auto trigger enable 
   //ADCSRA |= (1 << ADIE); // enable interrupt 
   //ADCSRA |= (1 << ADEN); // Enables ADC 
   //ADCSRA |= (1 << ADSC); // Start Conversion is enabled 
//   ADCSRB = (1 << ADTS2); // default is zero -> free running mode 

   ADCSRA= 
      (1<<ADEN)|//converter enable 
      (0<<ADSC)|//dont start conversion yet 
      (0<<ADATE)|//no ext triggering 
      (0<<ADIF)|//clear flag, stop pending conv 
      (1<<ADIE)|// interrupt enable 
      (1<<ADPS2)| 
      (1<<ADPS1)| 
      (1<<ADPS0);// ADC Prescaler Select Bits: Division Factor 128 
   ADMUX= 
      (0<<REFS1)| 
      (1<<REFS0)|//AVCC
      (0<<ADLAR)|//right-aligned 
      (0<<MUX3)| 
      (0<<MUX2)| 
      (0<<MUX1)| 
      (0<<MUX0);//channel 0, single-ended 
 
}
 

ISR(ADC_vect) { 
   const unsigned char adcl=ADCL; 
   const unsigned char adch=ADCH; 
   ADC_avg_value[ADC_channel][ADC_avg_pos] = 
   			ADC_recent_value[ADC_channel]=((unsigned)adch<<8) | adcl;
	
  
} 

void ADC_capture() 
{ 
   unsigned char channel; 
   // Capture:
   //ADC_channel = 0;
   //ADMUX = (0 << REFS1) | (1 << REFS0) | ADC_channel;
   //ADCSRA |= (1<<ADSC);
   //sleep_cpu();
   for(channel=0; channel<ADC_maxchannel; channel++) 
   { 
      ADC_channel=channel; 
      ADMUX = 
	  	(0<<REFS1)| 
	  	(1<<REFS0)|//AVCC
		channel; 
      //ADCSRA|=(1<<ADSC)|(1<<ADIE);// Sleeping will start conversion automagically!
	  SMCR |= (1<<SE);
      sleep_mode(); 
	  SMCR &= ~(1<<SE);
   
   }
   
} 

int calc_chan_avg(unsigned char channel) {
	unsigned int val = 0;
	for(unsigned char i = 0; i < ADC_avg_max; i++) {
		val += ADC_avg_value[channel][i];
	}
	return val/ADC_avg_max;
}

int main (void) 
{ 


   
   USART_Init();
   unsigned char rcv;
   while(glove_state != GLOVE_STATE_STREAM) {
   	rcv = USART_Receive();
	switch(rcv) {
		case 'n':
			USART_Transmitbuf("?n 6\r\n", 6);
			USART_Transmit(0);
			break;
		case 's':
			glove_state = GLOVE_STATE_STREAM;
			break;
		default:
			break;
	}
   }


   ADC_Init();
   sei();
   SMCR |= (0<<SM0) | (1<<SE);
   unsigned char channel;
   char buffer[100];
   unsigned char index = 0;
   for(;;) { 
   		
   		for(unsigned char pos = 0; pos < ADC_avg_max; pos++) {
			ADC_avg_pos = pos;
	   		ADC_capture();
			
			
			index = 0;
			buffer[index++] = 's';
			for(channel = 0; channel < ADC_maxchannel; channel++) {
	        	index += sprintf(buffer+index, " %4d",calc_chan_avg(channel));
	 		}
			buffer[index++] = '\r';
			buffer[index++] = '\n';
			buffer[index++] = 0;
	        USART_Transmitbuf(buffer, index);
			while ( !(UCSR0A & (1<<TXC0)) );   

		}
   } 
} 
