import math
from mathutils import Vector
from mathutils import Matrix
import bpy

def run_deform():
    gd = deform.GaussianDeform()
    gd.deform()

def gaussian(x, mu, sigma):
	return (1.0/math.sqrt(2.0*math.pi*sigma**2))*math.exp(-((x-mu)**2)/(2.0*sigma**2))
	
def gaussian2d(x, y, mux, muy, sigmax, sigmay):
	return (1.0/math.sqrt(2.0*math.pi*sigmax*sigmay))*math.exp(-((x-mux)**2)/(2.0*sigmax**2) - ((y-muy)**2/(2.0*sigmay**2)))
	
def gaussian2dmod(x, y, mux, muy, sigmax, sigmay):
	return math.exp(-((x-mux)**2)/(2.0*sigmax**2) - ((y-muy)**2/(2.0*sigmay**2)))

class GaussianDeform:
    def __init__(self, key="GridMesh"):
        self.key = key
        self.radius = 0.4
    
    def deform(self):
        mesh = bpy.data.objects[self.key]
        bones = bpy.data.objects["Armature"].pose.bones
        bonematrix = bpy.data.objects["Armature"].matrix
        fingertips = ["Bone_R.thumb.distal", "Bone_R.index.distal", "Bone_R.middle.distal", "Bone_R.ring.distal", "Bone_R.pinky.distal"]
        fingertips = ["Bone_R.index.distal"]
        for tip in fingertips:
            worldtail = bonematrix*bones[tip].tail
            worldhead = bonematrix*bones[tip].head
            normal = worldtail - worldhead
            normal /= normal.magnitude
            self.simpleDeform(worldtail, normal, mesh)
    
    def meshVertexLocalToWorld(mesh, i):
        return mesh.matrix * mesh.data.verts[i].co
        
    def simpleDeform(self, fingertip,normal,mesh):
        for vert in mesh.data.verts:
            vertworld = mesh.matrix * vert.co
            diff = vertworld - fingertip
            if diff.magnitude < self.radius:
                repel = 0.1*gaussian2dmod(vertworld.x,vertworld.y, fingertip.x, fingertip.y, 0.05, 0.05)
                if vert.co.z > -repel:
                    vert.co.z = -repel
    
    def directionalDeform(self, fingertip, normal, mesh):
        perp = Vector((-normal.y, normal.x, 0))
        perp2 = perp.cross(normal)
        tr = Matrix( (perp.x, perp.y, perp.z), 
                     (perp2.x, perp2.y, perp2.z), 
                     (normal.x, normal.y, normal.z))
        inv = tr.copy()
        inv.invert()
        meshinv = mesh.matrix.copy()
        meshinv.invert()
        for vert in mesh.data.verts:
            vertworld = mesh.matrix * vert.co
            diff = vertworld - fingertip
            if diff.magnitude < self.radius:
                moved = tr*diff
                moved.z += 0.1*gaussian2dmod(moved.x, moved.y, 0.0, 0.0, 0.025, 0.025)
                b = inv*moved + fingertip
                vert.co = meshinv*b

                 


    
