from python_vrpn import vrpn_Analog
import threading

class Glove:
    def __init__(self):
        pass

class CyberGlove(Glove):
    def __init__(self, bones):
        self.calibration_data_rest = [41.00, 70.00, 97.00, 21.00, 39.00, 60.00, 83.00, 
                                       68.00, 64.00, 55.00, 110.00, 34.00, 74.00, 59.00, 
                                       83.00, 35.00, 71.00, 88.00, 37.00, 65.00, 179.00, 71.00]
        self.calibration_data_flex = [50.00, 144.00, 193.00, 29.00, 178.00, 170.00, 
                                        138.00, 151.00, 159.00, 132.00, 145.00, 126.00, 163.00, 
                                        124.00, 103.00, 138.00, 156.00, 148.00, 81.00, 15.00, 151.00, 62.00]
        self.calibration_offset = [0.0 for i in range(22)]
        self.calibration_scale = [0.0 for i in range(22)]
        self.bones = bones
        self.vrpn_aremote = vrpn_Analog.vrpn_Analog_Remote("CyberGlove@localhost")
        vrpn_Analog.register_analog_change_handler(self.handle_analog)
        vrpn_Analog.vrpn_Analog_Remote.register_change_handler(self.vrpn_aremote,None,vrpn_Analog.get_analog_change_handler())
        self.calibrate()
        self.angles = [0.0 for i in range(22)]
    def calibrate(self):
        for i in range(len(self.calibration_data_rest)):
            self.calibration_offset[i] = self.calibration_data_flex[i] - self.calibration_data_rest[i]
            self.calibration_scale[i] = (self.calibration_data_flex[i] - self.calibration_data_rest[i]) / 0.4
    def handle_analog(self, userdata, a):
        for i in range(len(self.angles)):
            self.angles[i] = (a[i+1] - self.calibration_offset[i])/self.calibration_scale[i] # First a record is numchannels
        if not self.bones:
            print(self.angles)
            return
        self.bones["Bone_R.thumb.proximal"].rotation_quaternion[1] = self.angles[1]
        self.bones["Bone_R.thumb.intermediate"].rotation_quaternion[1] = self.angles[2]
        self.bones["Bone_R.thumb.distal"].rotation_quaternion[1] = self.angles[3]
        
        self.bones["Bone_R.index.proximal"].rotation_quaternion[1] = self.angles[4]
        self.bones["Bone_R.index.intermediate"].rotation_quaternion[1] = self.angles[5]
        self.bones["Bone_R.index.distal"].rotation_quaternion[1] = self.angles[6]
        
        self.bones["Bone_R.middle.proximal"].rotation_quaternion[1] = self.angles[8]
        self.bones["Bone_R.middle.intermediate"].rotation_quaternion[1] = self.angles[9]
        self.bones["Bone_R.middle.distal"].rotation_quaternion[1] = self.angles[10]
        
        self.bones["Bone_R.ring.proximal"].rotation_quaternion[1] = self.angles[12]
        self.bones["Bone_R.ring.intermediate"].rotation_quaternion[1] = self.angles[13]
        self.bones["Bone_R.ring.distal"].rotation_quaternion[1] = self.angles[14]
        
        self.bones["Bone_R.pinky.proximal"].rotation_quaternion[1] = self.angles[16]
        self.bones["Bone_R.pinky.intermediate"].rotation_quaternion[1] = self.angles[17]
        self.bones["Bone_R.pinky.distal"].rotation_quaternion[1] = self.angles[18]
        
        
    def update(self):
        vrpn_Analog.vrpn_Analog_Remote.mainloop(self.vrpn_aremote)
        
class SuperGlove(Glove):
    def __init__(self, bones):
        self.calibration_data_rest = [180.0, 180.0, 180.0, 1023.0, 1023.0, 1023.0]
        self.calibration_data_flex = [500.0, 500.0, 500.0, 500.0, 500.0, 500.0]
        self.calibration_offset = [0.0 for i in range(6)]
        self.calibration_scale = [0.0 for i in range(6)]
        self.bones = bones
        self.vrpn_aremote = vrpn_Analog.vrpn_Analog_Remote("CyberGlove@localhost")
        vrpn_Analog.register_analog_change_handler(self.handle_analog)
        vrpn_Analog.vrpn_Analog_Remote.register_change_handler(self.vrpn_aremote,None,vrpn_Analog.get_analog_change_handler())
        self.calibrate()
        self.angles = [0.0 for i in range(6)]
    def calibrate(self):
        for i in range(len(self.calibration_data_rest)):
            self.calibration_offset[i] = self.calibration_data_flex[i] - self.calibration_data_rest[i]
            self.calibration_scale[i] = (self.calibration_data_flex[i] - self.calibration_data_rest[i]) / 0.4
    def handle_analog(self, userdata, a):
        for i in range(len(self.angles)):
            self.angles[i] = (a[i+1] - self.calibration_offset[i])/self.calibration_scale[i] # First a record is numchannels
        if not self.bones:
            return
        self.bones["Bone_L.thumb.proximal"].rotation_quaternion[1] = self.angles[1]
        self.bones["Bone_L.index.proximal"].rotation_quaternion[1] = self.angles[2]
        self.bones["lBone_L.index.intermediate"].rotation_quaternion[1] = self.angles[0]
        
        
    def update(self):
        vrpn_Analog.vrpn_Analog_Remote.mainloop(self.vrpn_aremote)
        
        
