from mathutils import Vector
import time
import threading
class SpringMesh:
    def __init__(self, mesh):
        self.faces = mesh.data.faces
        self.verts = mesh.data.verts
        self.restLengths = {}#[[0.0 for i in range(len(self.verts))] for j in range(len(self.verts))]
        self.springConstants = {}#[[.5 for i in range(len(self.verts))] for j in range(len(self.verts))]
        self.dampingConstants = {}#[[.0 for i in range(len(self.verts))] for j in range(len(self.verts))]
        self.springConstant = 1.0
        self.dampingConstant = 0.1
        self.externalForces = [Vector() for i in range(len(self.verts))]
        self.internalForces = [Vector() for i in range(len(self.verts))]
        self.vels = [Vector() for i in range(len(self.verts))]
        self.accels = [Vector() for i in range(len(self.verts))]
        self.timestep = 0.01
        self.initSprings()
    def initSprings(self):
        for face in self.faces:
            for i in face.verts:
                for j in face.verts:
                    if i == j: continue
                    key = "%d %d" % (i,j)
                    okey = "%d %d" % (j,i)
                    if key not in self.restLengths and okey not in self.restLengths:
                        self.restLengths[key] = (self.verts[i].co - self.verts[j].co).magnitude
    def computeInternalForces(self):
        for key in self.restLengths:
            split = key.split(" ")
            i,j = int(split[0]),int(split[1])
            restLength = self.restLengths[key]
            diff = (self.verts[i].co - self.verts[j].co)
            diff = diff*((diff.magnitude - restLength)/restLength)
            f = -self.springConstant*diff -self.dampingConstant*self.vels[i]
            self.internalForces[i] += f
                        
    def applyForces(self):
        for i in range(len(self.verts)):
            self.accels[i] = self.internalForces[i] + self.externalForces[i]
            self.vels[i] = self.accels[i]*self.timestep
            self.verts[i].co += self.vels[i]*self.timestep + 0.5*self.accels[i]*self.timestep**2

    def run(self):
        self.stop = False
        def mainloop():
            while not self.stop:
                self.computeInternalForces()
                self.applyForces()
                time.sleep(1.0/20.0)
        thread = threading.Thread(target=mainloop)
        thread.start()
        #mainloop()
                

