
#include "vrpn_Analog.h"
#include "vrpn_CyberGlove.h"
#include "vrpn_Connection.h"
#include "vrpn_Serial.h"

#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <string.h>


vrpn_CyberGlove::vrpn_CyberGlove(const char * name,
                                        vrpn_Connection * c,
                                        const char * p_device, const char * p_baud) :
	vrpn_Analog (name, c)
{
    this->dev_fd = vrpn_open_commport(p_device, atol(p_baud));
    if (this->dev_fd < 0) {
	    fprintf(stderr, "vrpn_CyberGlove: Could not open %s\n", p_device);
	    return;
    } 
    vrpn_flush_input_buffer(this->dev_fd);
    vrpn_write_characters(this->dev_fd, (const unsigned char*)"?n", strlen("?n"));

    struct timeval wait;
    wait.tv_sec = 1;
	vrpn_read_available_characters(this->dev_fd, this->buffer, 8, &wait);
	nSensors = atoi((const char*)(&buffer[3]));
    printf("CyberGlove: Channels: %d %s\n", nSensors, buffer);

	this->setNumChannels(nSensors);
	
	// Start ASCII stream mode
	vrpn_write_characters(this->dev_fd, (const unsigned char*)"s", strlen("s"));
}

vrpn_CyberGlove::~vrpn_CyberGlove(void) {
	vrpn_close_commport(this->dev_fd);
}


void vrpn_CyberGlove::report_changes (vrpn_uint32 class_of_service)
{
  vrpn_Analog::timestamp = timestamp;
  vrpn_Analog::report_changes(class_of_service);
}

void vrpn_CyberGlove::report (vrpn_uint32 class_of_service)
{
	vrpn_Analog::timestamp = timestamp;
	vrpn_Analog::report(class_of_service);
}


vrpn_int32 vrpn_CyberGlove::setNumChannels (vrpn_int32 sizeRequested) {
  if (sizeRequested < 0) sizeRequested = 0;
  if (sizeRequested > vrpn_CHANNEL_MAX) sizeRequested = vrpn_CHANNEL_MAX;
  num_channel = sizeRequested;
  return num_channel;
}


void vrpn_CyberGlove::mainloop (void) {
	//fprintf(stderr, "vrpn_Cyberglove::mainloop");
	server_mainloop();
	vrpn_gettimeofday(&timestamp, NULL);
	vrpn_read_available_characters(this->dev_fd, this->buffer, 255, NULL);
	this->buffer[255] = NULL; // Prevent reading past end
	unsigned char * ptr = this->buffer;
	unsigned char character = 0;
	int position = -1;
	int n = 0;
	while(ptr < buffer+255) {
	    n = 0;
        if(character == 's') {
            position = 0;
            character = 0;
            report_changes();
        } else if(position != -1 && position < num_channel) {
            //n = sscanf((const char*)ptr, "%d", &this->value[position]);
            value[position] =  strtol((const char *)ptr, (char**)&ptr, 10);
            if(value[position] != 0)
                channel[position] = value[position];
            //channel[position] = value[position] = atoi((const char *)this->str);
            position++;
        } else {
            character = *ptr;
            position = -1;
            ptr++;
        }
        //sscanf((const char *)ptr, "%s", this->str);
	}
	/*
	if(buffer[0] != 's') {
	    return;
	 }
	char * token = strtok((char*)buffer, " ");
	for (int i=0; i<nSensors; i++) {
	    token = strtok(NULL, " ");
	    if(token != NULL)
	        channel[i] = value[i] = atoi(token);
	}
	*/
	
}
