#ifndef VRPN_CYBERGLOVE_H
#define VRPN_CYBERGLOVE_H

#include "vrpn_Connection.h"
#include "vrpn_Analog.h"

/*
0.   thumb rotation/TMJ (angle of thumb rotating across palm)
1.   thumb MPJ (joint where the thumb meets the palm)
2.   thumb IJ (outer thumb joint)
3.   thumb abduction (angle between thumb and index finger)
4.   index MPJ (joint where the index meets the palm)
5.   index PIJ (joint second from finger tip)
6.*  index DIJ (joint closest to finger tip)
7.** index abduction (sideways motion of index finger)
8.   middle MPJ
9.   middle PIJ
10.* middle DIJ
11.  middle-index abd'n (angle between middle and index fingers)
12.  ring MPJ
13.  ring PIJ
14.* ring DIJ
15.  ring-middle abduction (angle between ring and middle fingers)
16.  pinkie MPJ
17.  pinkie PIJ
18.* pinkie DIJ
19.  pinkie-ring abduction (angle between pinkie and ring finger)
20.  palm arch (causes pinkie to rotate across palm)
21.  wrist pitch (flexion/extension)
22.  wrist yaw (abduction/adduction)
*/

class VRPN_API vrpn_CyberGlove : public vrpn_Analog {

  public:
	vrpn_CyberGlove(const char * name, vrpn_Connection * c);
    vrpn_CyberGlove(const char * name, vrpn_Connection * c,	const char * device, const char * baud);
	~vrpn_CyberGlove(void);

    /// Sets the size of the array;  returns the size actually set.
    /// (May be clamped to vrpn_CHANNEL_MAX)   
    /// This should be used before mainloop is ever called.
    vrpn_int32 setNumChannels (vrpn_int32 sizeRequested);

	virtual void mainloop();
protected:
	const char * _device;
	const char * _baud;
	int dev_fd;
	
	int		nSensors;
	static const int buf_size = 256;
    unsigned char	buffer[256];
    unsigned char str[20];
	int		value[22];
	
	struct timeval timestamp;
	
		/// send report iff changed
        virtual void report_changes
                   (vrpn_uint32 class_of_service = vrpn_CONNECTION_LOW_LATENCY);
        /// send report whether or not changed
        virtual void report
                   (vrpn_uint32 class_of_service = vrpn_CONNECTION_LOW_LATENCY);

    void    UpdateData();     		    
	void	Wait(double t);
	

};

#endif
